# SPDX-License-Identifier: BSD-2-Clause

# Copyright (c) 2024 Phil Thompson <phil@riverbankcomputing.com>


# The minimum required version of setuptools.  This is the earliest version
# that supports PEP 625.  Remember to update pyproject.toml in the root
# directory.
MINIMUM_SETUPTOOLS = '69.5'

# The oldest supported minor version of Python v3.  Remember to update
# pyproject.toml in the root directory.
OLDEST_SUPPORTED_MINOR = 9
